@@ -1,4 +1,5 @@
 """Convenience functions for storing and loading data."""
+
 import itertools
 import os
 import re
@@ -30,6 +31,8 @@ StoreLike = Union[BaseStore, MutableMapping, str, None]
 
 _builtin_open = open  # builtin open is later shadowed by a local open function
 
+__doctest_requires__ = {("*"): ["numpy>=2.2"]}
+
 
 def _check_and_update_path(store: BaseStore, path):
     if getattr(store, "_store_version", 2) > 2 and not path:
@@ -54,6 +57,11 @@ def open(store: StoreLike = None, mode: str = "a", *, zarr_version=None, path=No
         The zarr protocol version to use. The default value of None will attempt
         to infer the version from `store` if possible, otherwise it will fall
         back to 2.
+
+        .. warning:: `zarr_version=3` is currently using the experimental Zarr V3
+          implementation. This implementation is not in sync with the final specification
+          and will be replaced with a spec compliant version in the version 3.0.
+
     path : str or None, optional
         The path within the store to open.
     **kwargs
@@ -149,6 +157,11 @@ def save_array(store: StoreLike, arr, *, zarr_version=None, path=None, **kwargs)
         The zarr protocol version to use when saving. The default value of None
         will attempt to infer the version from `store` if possible, otherwise
         it will fall back to 2.
+
+        .. warning:: `zarr_version=3` is currently using the experimental Zarr V3
+          implementation. This implementation is not in sync with the final specification
+          and will be replaced with a spec compliant version in the version 3.0.
+
     path : str or None, optional
         The path within the store where the array will be saved.
     kwargs
@@ -163,13 +176,13 @@ def save_array(store: StoreLike, arr, *, zarr_version=None, path=None, **kwargs)
         >>> arr = np.arange(10000)
         >>> zarr.save_array('data/example.zarr', arr)
         >>> zarr.load('data/example.zarr')
-        array([   0,    1,    2, ..., 9997, 9998, 9999])
+        array([   0,    1,    2, ..., 9997, 9998, 9999], shape=(10000,))
 
     Save an array to a single file (uses a :class:`ZipStore`)::
 
         >>> zarr.save_array('data/example.zip', arr)
         >>> zarr.load('data/example.zip')
-        array([   0,    1,    2, ..., 9997, 9998, 9999])
+        array([   0,    1,    2, ..., 9997, 9998, 9999], shape=(10000,))
 
     """
     may_need_closing = _might_close(store)
@@ -199,6 +212,11 @@ def save_group(store: StoreLike, *args, zarr_version=None, path=None, **kwargs):
         The zarr protocol version to use when saving. The default value of None
         will attempt to infer the version from `store` if possible, otherwise
         it will fall back to 2.
+
+        .. warning:: `zarr_version=3` is currently using the experimental Zarr V3
+          implementation. This implementation is not in sync with the final specification
+          and will be replaced with a spec compliant version in the version 3.0.
+
     path : str or None, optional
         Path within the store where the group will be saved.
     kwargs
@@ -218,9 +236,9 @@ def save_group(store: StoreLike, *args, zarr_version=None, path=None, **kwargs):
         >>> loader
         <LazyLoader: arr_0, arr_1>
         >>> loader['arr_0']
-        array([   0,    1,    2, ..., 9997, 9998, 9999])
+        array([   0,    1,    2, ..., 9997, 9998, 9999], shape=(10000,))
         >>> loader['arr_1']
-        array([10000,  9999,  9998, ...,     3,     2,     1])
+        array([10000,  9999,  9998, ...,     3,     2,     1], shape=(10000,))
 
     Save several arrays using named keyword arguments::
 
@@ -229,9 +247,9 @@ def save_group(store: StoreLike, *args, zarr_version=None, path=None, **kwargs):
         >>> loader
         <LazyLoader: bar, foo>
         >>> loader['foo']
-        array([   0,    1,    2, ..., 9997, 9998, 9999])
+        array([   0,    1,    2, ..., 9997, 9998, 9999], shape=(10000,))
         >>> loader['bar']
-        array([10000,  9999,  9998, ...,     3,     2,     1])
+        array([10000,  9999,  9998, ...,     3,     2,     1], shape=(10000,))
 
     Store several arrays in a single zip file (uses a :class:`ZipStore`)::
 
@@ -240,9 +258,9 @@ def save_group(store: StoreLike, *args, zarr_version=None, path=None, **kwargs):
         >>> loader
         <LazyLoader: bar, foo>
         >>> loader['foo']
-        array([   0,    1,    2, ..., 9997, 9998, 9999])
+        array([   0,    1,    2, ..., 9997, 9998, 9999], shape=(10000,))
         >>> loader['bar']
-        array([10000,  9999,  9998, ...,     3,     2,     1])
+        array([10000,  9999,  9998, ...,     3,     2,     1], shape=(10000,))
 
     Notes
     -----
@@ -258,7 +276,7 @@ def save_group(store: StoreLike, *args, zarr_version=None, path=None, **kwargs):
     try:
         grp = _create_group(_store, path=path, overwrite=True, zarr_version=zarr_version)
         for i, arr in enumerate(args):
-            k = "arr_{}".format(i)
+            k = f"arr_{i}"
             grp.create_dataset(k, data=arr, overwrite=True, zarr_version=zarr_version)
         for k, arr in kwargs.items():
             grp.create_dataset(k, data=arr, overwrite=True, zarr_version=zarr_version)
@@ -281,6 +299,11 @@ def save(store: StoreLike, *args, zarr_version=None, path=None, **kwargs):
         The zarr protocol version to use when saving. The default value of None
         will attempt to infer the version from `store` if possible, otherwise
         it will fall back to 2.
+
+        .. warning:: `zarr_version=3` is currently using the experimental Zarr V3
+          implementation. This implementation is not in sync with the final specification
+          and will be replaced with a spec compliant version in the version 3.0.
+
     path : str or None, optional
         The path within the group where the arrays will be saved.
     kwargs
@@ -295,13 +318,13 @@ def save(store: StoreLike, *args, zarr_version=None, path=None, **kwargs):
         >>> arr = np.arange(10000)
         >>> zarr.save('data/example.zarr', arr)
         >>> zarr.load('data/example.zarr')
-        array([   0,    1,    2, ..., 9997, 9998, 9999])
+        array([   0,    1,    2, ..., 9997, 9998, 9999], shape=(10000,))
 
     Save an array to a Zip file (uses a :class:`ZipStore`)::
 
         >>> zarr.save('data/example.zip', arr)
         >>> zarr.load('data/example.zip')
-        array([   0,    1,    2, ..., 9997, 9998, 9999])
+        array([   0,    1,    2, ..., 9997, 9998, 9999], shape=(10000,))
 
     Save several arrays to a directory on the file system (uses a
     :class:`DirectoryStore` and stores arrays in a group)::
@@ -315,9 +338,9 @@ def save(store: StoreLike, *args, zarr_version=None, path=None, **kwargs):
         >>> loader
         <LazyLoader: arr_0, arr_1>
         >>> loader['arr_0']
-        array([   0,    1,    2, ..., 9997, 9998, 9999])
+        array([   0,    1,    2, ..., 9997, 9998, 9999], shape=(10000,))
         >>> loader['arr_1']
-        array([10000,  9999,  9998, ...,     3,     2,     1])
+        array([10000,  9999,  9998, ...,     3,     2,     1], shape=(10000,))
 
     Save several arrays using named keyword arguments::
 
@@ -326,9 +349,9 @@ def save(store: StoreLike, *args, zarr_version=None, path=None, **kwargs):
         >>> loader
         <LazyLoader: bar, foo>
         >>> loader['foo']
-        array([   0,    1,    2, ..., 9997, 9998, 9999])
+        array([   0,    1,    2, ..., 9997, 9998, 9999], shape=(10000,))
         >>> loader['bar']
-        array([10000,  9999,  9998, ...,     3,     2,     1])
+        array([10000,  9999,  9998, ...,     3,     2,     1], shape=(10000,))
 
     Store several arrays in a single zip file (uses a :class:`ZipStore`)::
 
@@ -337,9 +360,9 @@ def save(store: StoreLike, *args, zarr_version=None, path=None, **kwargs):
         >>> loader
         <LazyLoader: bar, foo>
         >>> loader['foo']
-        array([   0,    1,    2, ..., 9997, 9998, 9999])
+        array([   0,    1,    2, ..., 9997, 9998, 9999], shape=(10000,))
         >>> loader['bar']
-        array([10000,  9999,  9998, ...,     3,     2,     1])
+        array([10000,  9999,  9998, ...,     3,     2,     1], shape=(10000,))
 
     See Also
     --------
@@ -394,6 +417,11 @@ def load(store: StoreLike, zarr_version=None, path=None):
         The zarr protocol version to use when loading. The default value of
         None will attempt to infer the version from `store` if possible,
         otherwise it will fall back to 2.
+
+        .. warning:: `zarr_version=3` is currently using the experimental Zarr V3
+          implementation. This implementation is not in sync with the final specification
+          and will be replaced with a spec compliant version in the version 3.0.
+
     path : str or None, optional
         The path within the store from which to load.
 
@@ -498,7 +526,7 @@ class _LogWriter:
             self.log_file = log
         else:
             raise TypeError(
-                "log must be a callable function, file path or " "file-like object, found %r" % log
+                f"log must be a callable function, file path or file-like object, found {log!r}"
             )
 
     def __enter__(self):
@@ -525,9 +553,9 @@ def _log_copy_summary(log, dry_run, n_copied, n_skipped, n_bytes_copied):
         message = "dry run: "
     else:
         message = "all done: "
-    message += "{:,} copied, {:,} skipped".format(n_copied, n_skipped)
+    message += f"{n_copied:,} copied, {n_skipped:,} skipped"
     if not dry_run:
-        message += ", {:,} bytes copied".format(n_bytes_copied)
+        message += f", {n_bytes_copied:,} bytes copied"
     log(message)
 
 
@@ -656,9 +684,7 @@ def copy_store(
     # check if_exists parameter
     valid_if_exists = ["raise", "replace", "skip"]
     if if_exists not in valid_if_exists:
-        raise ValueError(
-            "if_exists must be one of {!r}; found {!r}".format(valid_if_exists, if_exists)
-        )
+        raise ValueError(f"if_exists must be one of {valid_if_exists!r}; found {if_exists!r}")
 
     # setup counting variables
     n_copied = n_skipped = n_bytes_copied = 0
@@ -719,20 +745,20 @@ def copy_store(
             if if_exists != "replace":
                 if dest_key in dest:
                     if if_exists == "raise":
-                        raise CopyError("key {!r} exists in destination".format(dest_key))
+                        raise CopyError(f"key {dest_key!r} exists in destination")
                     elif if_exists == "skip":
                         do_copy = False
 
             # take action
             if do_copy:
-                log("copy {}".format(descr))
+                log(f"copy {descr}")
                 if not dry_run:
                     data = source[source_key]
                     n_bytes_copied += buffer_size(data)
                     dest[dest_key] = data
                 n_copied += 1
             else:
-                log("skip {}".format(descr))
+                log(f"skip {descr}")
                 n_skipped += 1
 
         # log a final message with a summary of what happened
@@ -743,7 +769,7 @@ def copy_store(
 
 def _check_dest_is_group(dest):
     if not hasattr(dest, "create_dataset"):
-        raise ValueError("dest must be a group, got {!r}".format(dest))
+        raise ValueError(f"dest must be a group, got {dest!r}")
 
 
 def copy(
@@ -909,11 +935,9 @@ def _copy(log, source, dest, name, root, shallow, without_attrs, if_exists, dry_
     # check if_exists parameter
     valid_if_exists = ["raise", "replace", "skip", "skip_initialized"]
     if if_exists not in valid_if_exists:
-        raise ValueError(
-            "if_exists must be one of {!r}; found {!r}".format(valid_if_exists, if_exists)
-        )
+        raise ValueError(f"if_exists must be one of {valid_if_exists!r}; found {if_exists!r}")
     if dest_h5py and if_exists == "skip_initialized":
-        raise ValueError("{!r} can only be used when copying to zarr".format(if_exists))
+        raise ValueError(f"{if_exists!r} can only be used when copying to zarr")
 
     # determine name to copy to
     if name is None:
@@ -933,9 +957,7 @@ def _copy(log, source, dest, name, root, shallow, without_attrs, if_exists, dry_
         exists = dest is not None and name in dest
         if exists:
             if if_exists == "raise":
-                raise CopyError(
-                    "an object {!r} already exists in destination " "{!r}".format(name, dest.name)
-                )
+                raise CopyError(f"an object {name!r} already exists in destination {dest.name!r}")
             elif if_exists == "skip":
                 do_copy = False
             elif if_exists == "skip_initialized":
@@ -946,7 +968,7 @@ def _copy(log, source, dest, name, root, shallow, without_attrs, if_exists, dry_
         # take action
         if do_copy:
             # log a message about what we're going to do
-            log("copy {} {} {}".format(source.name, source.shape, source.dtype))
+            log(f"copy {source.name} {source.shape} {source.dtype}")
 
             if not dry_run:
                 # clear the way
@@ -1014,7 +1036,7 @@ def _copy(log, source, dest, name, root, shallow, without_attrs, if_exists, dry_
             n_copied += 1
 
         else:
-            log("skip {} {} {}".format(source.name, source.shape, source.dtype))
+            log(f"skip {source.name} {source.shape} {source.dtype}")
             n_skipped += 1
 
     elif root or not shallow:
@@ -1025,16 +1047,14 @@ def _copy(log, source, dest, name, root, shallow, without_attrs, if_exists, dry_
         exists_array = dest is not None and name in dest and hasattr(dest[name], "shape")
         if exists_array:
             if if_exists == "raise":
-                raise CopyError(
-                    "an array {!r} already exists in destination " "{!r}".format(name, dest.name)
-                )
+                raise CopyError(f"an array {name!r} already exists in destination {dest.name!r}")
             elif if_exists == "skip":
                 do_copy = False
 
         # take action
         if do_copy:
             # log action
-            log("copy {}".format(source.name))
+            log(f"copy {source.name}")
 
             if not dry_run:
                 # clear the way
@@ -1077,7 +1097,7 @@ def _copy(log, source, dest, name, root, shallow, without_attrs, if_exists, dry_
             n_copied += 1
 
         else:
-            log("skip {}".format(source.name))
+            log(f"skip {source.name}")
             n_skipped += 1
 
     return n_copied, n_skipped, n_bytes_copied
@@ -1326,7 +1346,7 @@ def open_consolidated(store: StoreLike, metadata_key=".zmetadata", mode="r+", **
         store, storage_options=kwargs.get("storage_options"), mode=mode, zarr_version=zarr_version
     )
     if mode not in {"r", "r+"}:
-        raise ValueError("invalid mode, expected either 'r' or 'r+'; found {!r}".format(mode))
+        raise ValueError(f"invalid mode, expected either 'r' or 'r+'; found {mode!r}")
 
     path = kwargs.pop("path", None)
     if store._store_version == 2:
